<?php
namespace DEPOClient\Entities;

use DEPOClient\Exceptions\InvalidEntityException;

abstract class BaseEntity {
	/** @var array Required fields of the entity. Override in descendants. */
	protected $_required = [];

	/** @var array Fields used in DEPO api. Override in descendants. */
	protected $_DEPO_fields = [];

	/**
	 * Checks if required fields are set.
	 * Calls _check method which can contain additional validation.
	 * @throws InvalidEntityException
	 */
	public function checkFields() {
		foreach($this->getRequiredFields() as $req) {
			if (empty($this->{$req})) {
				throw new InvalidEntityException('Missing field '.$req);
			}
		}
		$this->_check();
	}

	/**
	 * Override in descendants if needed
	 * @throws InvalidEntityException
	 */
	protected function _check() {}

	/**
	 * 
	 * @return array Required field names.
	 */
	public function getRequiredFields() {
		return $this->_required;
	}

	/**
	 * Gets identifier of entity.
	 * If this is null, it means that entity is not yet in the system.
	 * @return mixed
	 */
	abstract public function getId();

	/**
	 * Returns entity encoded as json.
	 * With that should be used with DEPO api.
	 * @return mixed
	 */
	public function asDEPOJson() {
		return json_encode($this->asDEPOArray(), JSON_UNESCAPED_UNICODE);
	}

	/**
	 * Returns array with fields of this object
	 * that should be used with DEPO api.
	 * @return array
	 */
	public function asDEPOArray() {
		$data = [];
		foreach ($this->_DEPO_fields as $k) {
			$data[$k] = $this->{$k};
		}
		return $data;
	}

	/**
	 * Fills class properties with data.
	 * @param $data
	 */
	public function fillFromArray($data) {
		foreach($data as $key => $value) {
			if (property_exists($this, $key)) {
				$this->{$key} = $value;
			}
		}
	}
}